@@ -1,64 +1,104 @@
 from __future__ import annotations
 
-import calendar
 import datetime
 import time
-from typing import TYPE_CHECKING, Any
+from abc import ABC, abstractmethod
+from typing import Any, cast
 
-from deprecated.sphinx import versionadded
+from deprecated.sphinx import versionadded, versionchanged
 
-from limits.typing import Dict, Optional, Tuple, Union
+from limits.typing import (
+    Dict,
+    MongoClient,
+    MongoCollection,
+    MongoDatabase,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+)
 
+from ..util import get_dependency
 from .base import MovingWindowSupport, Storage
 
-if TYPE_CHECKING:
-    import pymongo
 
-
-@versionadded(version="2.1")
-class MongoDBStorage(Storage, MovingWindowSupport):
+class MongoDBStorageBase(Storage, MovingWindowSupport, ABC):
     """
     Rate limit storage with MongoDB as backend.
 
     Depends on :pypi:`pymongo`.
     """
 
-    STORAGE_SCHEME = ["mongodb", "mongodb+srv"]
-    DEFAULT_OPTIONS: Dict[str, Union[int, str, bool]] = {
-        "serverSelectionTimeoutMS": 1000,
-        "connectTimeoutMS": 1000,
-    }
-    "Default options passed to :class:`~pymongo.mongo_client.MongoClient`"
-
     DEPENDENCIES = ["pymongo"]
 
     def __init__(
-        self, uri: str, database_name: str = "limits", **options: Union[int, str, bool]
+        self,
+        uri: str,
+        database_name: str = "limits",
+        counter_collection_name: str = "counters",
+        window_collection_name: str = "windows",
+        wrap_exceptions: bool = False,
+        **options: Union[int, str, bool],
     ) -> None:
         """
         :param uri: uri of the form ``mongodb://[user:password]@host:port?...``,
          This uri is passed directly to :class:`~pymongo.mongo_client.MongoClient`
         :param database_name: The database to use for storing the rate limit
          collections.
-        :param options: all remaining keyword arguments are merged with
-         :data:`DEFAULT_OPTIONS` and passed to the constructor of
-         :class:`~pymongo.mongo_client.MongoClient`
+        :param counter_collection_name: The collection name to use for individual counters
+         used in fixed window strategies
+        :param window_collection_name: The collection name to use for moving window storage
+        :param wrap_exceptions: Whether to wrap storage exceptions in
+         :exc:`limits.errors.StorageError` before raising it.
+        :param options: all remaining keyword arguments are passed to the
+         constructor of :class:`~pymongo.mongo_client.MongoClient`
         :raise ConfigurationError: when the :pypi:`pymongo` library is not available
         """
 
-        super().__init__(uri, **options)
-
+        super().__init__(uri, wrap_exceptions=wrap_exceptions, **options)
+        self._database_name = database_name
+        self._collection_mapping = {
+            "counters": counter_collection_name,
+            "windows": window_collection_name,
+        }
         self.lib = self.dependencies["pymongo"].module
+        self.lib_errors, _ = get_dependency("pymongo.errors")
+        self._storage_uri = uri
+        self._storage_options = options
+        self._storage: Optional[MongoClient] = None
 
-        mongo_opts = options.copy()
-        [mongo_opts.setdefault(k, v) for k, v in self.DEFAULT_OPTIONS.items()]
+    @property
+    def storage(self) -> MongoClient:
+        if self._storage is None:
+            self._storage = self._init_mongo_client(
+                self._storage_uri, **self._storage_options
+            )
+            self.__initialize_database()
+        return self._storage
 
-        self.storage: "pymongo.MongoClient" = self.lib.MongoClient(  # type: ignore[type-arg]
-            uri, **mongo_opts
-        )
-        self.counters = self.storage.get_database(database_name).counters
-        self.windows = self.storage.get_database(database_name).windows
-        self.__initialize_database()
+    @property
+    def _database(self) -> MongoDatabase:
+        return self.storage[self._database_name]
+
+    @property
+    def counters(self) -> MongoCollection:
+        return self._database[self._collection_mapping["counters"]]
+
+    @property
+    def windows(self) -> MongoCollection:
+        return self._database[self._collection_mapping["windows"]]
+
+    @abstractmethod
+    def _init_mongo_client(
+        self, uri: Optional[str], **options: Union[int, str, bool]
+    ) -> MongoClient:
+        raise NotImplementedError()
+
+    @property
+    def base_exceptions(
+        self,
+    ) -> Union[Type[Exception], Tuple[Type[Exception], ...]]:  # pragma: no cover
+        return self.lib_errors.PyMongoError  # type: ignore
 
     def __initialize_database(self) -> None:
         self.counters.create_index("expireAt", expireAfterSeconds=0)
@@ -81,21 +121,26 @@ class MongoDBStorage(Storage, MovingWindowSupport):
         self.counters.find_one_and_delete({"_id": key})
         self.windows.find_one_and_delete({"_id": key})
 
-    def get_expiry(self, key: str) -> int:
+    def get_expiry(self, key: str) -> float:
         """
         :param key: the key to get the expiry for
         """
         counter = self.counters.find_one({"_id": key})
-        expiry = counter["expireAt"] if counter else datetime.datetime.utcnow()
-
-        return calendar.timegm(expiry.timetuple())
+        return (
+            (counter["expireAt"] if counter else datetime.datetime.now())
+            .replace(tzinfo=datetime.timezone.utc)
+            .timestamp()
+        )
 
     def get(self, key: str) -> int:
         """
         :param key: the key to get the counter value for
         """
         counter = self.counters.find_one(
-            {"_id": key, "expireAt": {"$gte": datetime.datetime.utcnow()}},
+            {
+                "_id": key,
+                "expireAt": {"$gte": datetime.datetime.now(datetime.timezone.utc)},
+            },
             projection=["count"],
         )
 
@@ -111,7 +156,9 @@ class MongoDBStorage(Storage, MovingWindowSupport):
         :param expiry: amount in seconds for the key to expire in
         :param amount: the number to increment by
         """
-        expiration = datetime.datetime.utcnow() + datetime.timedelta(seconds=expiry)
+        expiration = datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(
+            seconds=expiry
+        )
 
         return int(
             self.counters.find_one_and_update(
@@ -155,7 +202,7 @@ class MongoDBStorage(Storage, MovingWindowSupport):
         except:  # noqa: E722
             return False
 
-    def get_moving_window(self, key: str, limit: int, expiry: int) -> Tuple[int, int]:
+    def get_moving_window(self, key: str, limit: int, expiry: int) -> Tuple[float, int]:
         """
         returns the starting point and the number of entries in the moving
         window
@@ -184,7 +231,7 @@ class MongoDBStorage(Storage, MovingWindowSupport):
                     {
                         "$group": {
                             "_id": "$_id",
-                            "max": {"$max": "$entries"},
+                            "min": {"$min": "$entries"},
                             "count": {"$sum": 1},
                         }
                     },
@@ -193,9 +240,9 @@ class MongoDBStorage(Storage, MovingWindowSupport):
         )
 
         if result:
-            return int(result[0]["max"]), result[0]["count"]
+            return result[0]["min"], result[0]["count"]
 
-        return int(timestamp), 0
+        return timestamp, 0
 
     def acquire_entry(self, key: str, limit: int, expiry: int, amount: int = 1) -> bool:
         """
@@ -215,15 +262,17 @@ class MongoDBStorage(Storage, MovingWindowSupport):
 
             updates["$set"] = {
                 "expireAt": (
-                    datetime.datetime.utcnow() + datetime.timedelta(seconds=expiry)
+                    datetime.datetime.now(datetime.timezone.utc)
+                    + datetime.timedelta(seconds=expiry)
                 )
             }
             updates["$push"]["entries"]["$each"] = [timestamp] * amount
             self.windows.update_one(
                 {
                     "_id": key,
-                    "entries.%d"
-                    % (limit - amount): {"$not": {"$gte": timestamp - expiry}},
+                    "entries.%d" % (limit - amount): {
+                        "$not": {"$gte": timestamp - expiry}
+                    },
                 },
                 updates,
                 upsert=True,
@@ -232,3 +281,17 @@ class MongoDBStorage(Storage, MovingWindowSupport):
             return True
         except self.lib.errors.DuplicateKeyError:
             return False
+
+
+@versionadded(version="2.1")
+@versionchanged(
+    version="3.14.0",
+    reason="Added option to select custom collection names for windows & counters",
+)
+class MongoDBStorage(MongoDBStorageBase):
+    STORAGE_SCHEME = ["mongodb", "mongodb+srv"]
+
+    def _init_mongo_client(
+        self, uri: Optional[str], **options: Union[int, str, bool]
+    ) -> MongoClient:
+        return cast(MongoClient, self.lib.MongoClient(uri, **options))
